//package org.jbpm.test.task;

import java.util.*;

import org.jbpm.api.*;
import org.jbpm.api.activity.*;
import org.jbpm.api.job.*;
import org.jbpm.api.task.*;
import org.jbpm.pvm.internal.env.*;

import org.jbpm.test.*;


public class SubTest extends JbpmTestCase {
    static final String XML_MAIN = ""
        + "<process key='main' name='main' xmlns='http://jbpm.org/4.3/jpdl'>"
        + "   <start g='5,23,48,48' name='start'>"
        + "      <transition g='141,46:-88,-15' name='open.case' to='Create sub-process'/>"
        + "   </start>"
        + "   <state g='306,193,148,52' name='Wait For Update'>"
        + "      <transition g='-23,-24' name='end' to='end1'/>"
        + "   </state>"
        + "   <custom g='239,26,138,52' name='Create sub-process' class='SubTest$CreateSubprocesses'>"
        + "      <transition g='484,52:-108,-17' name='to-split-paths' to='Wait For Update'/>"
        + "   </custom>"
        + "  <end g='566,493,48,48' name='end1'/>"
        + "</process>";

    static final String XML_SUB = ""
        + "<process name='test' key='test' xmlns='http://jbpm.org/4.3/jpdl'>"
        + "   <swimlane candidate-groups='#{taskAssignees}' name='assigned-group'/>"
        + "   <start name='start1' g='16,21,48,48'>"
        + "      <transition name='to task1' to='task1' g='-17,-20'/>"
        + "   </start>"
        + "   <end name='end1' g='219,17,48,48'/>"
        + "   <task name='task1' g='96,16,92,52' swimlane='assigned-group'>"
        + "      <transition name='to end1' to='end1' g='-20,-19'/>"
        + "   </task>"
        + "</process>";

    String deploymentId;

    protected void setUp() throws Exception {
        super.setUp();
        identityService.createUser("johndoe", "johndoe", "johndoe");
        identityService.createGroup("group");
        identityService.createMembership("johndoe", "group", "role");
        deploymentId = repositoryService.createDeployment()
                                        .addResourceFromString("main.jpdl.xml", XML_MAIN)
                                        .addResourceFromString("test.jpdl.xml", XML_SUB)
                                        .deploy();
    }

    protected void tearDown() throws Exception {
        identityService.deleteUser("johndoe");
        identityService.deleteGroup("group");
        repositoryService.deleteDeploymentCascade(deploymentId);
        super.tearDown();
    }

    public void testDefault() throws Exception {
        ProcessInstance processInstance = executionService
            .startProcessInstanceByKey("main");
    }

    public static class CreateSubprocesses implements ActivityBehaviour {
    public void execute(ActivityExecution activityExecution) {
        ExecutionService executionService = EnvironmentImpl.getFromCurrent(ExecutionService.class);
        ProcessInstance processInstance = executionService.startProcessInstanceByKey("test",
            Collections.singletonMap("taskAssignees", "group"));
        List<Task> taskList = taskService.findGroupTasks("johndoe");
        assertEquals(1, taskList.size());
    }
}
}
